@@ -26,6 +26,7 @@ module Data.Aeson.Types
     -- * Convenience types and functions
     , DotNetTime(..)
     , typeMismatch
+    , unexpected
     -- * Type conversion
     , Parser
     , Result(..)
@@ -34,9 +35,11 @@ module Data.Aeson.Types
     , parse
     , parseEither
     , parseMaybe
+    , parseFail
     , ToJSON(..)
     , KeyValue(..)
     , modifyFailure
+    , prependFailure
     , parserThrowError
     , parserCatchError
 
@@ -51,6 +54,12 @@ module Data.Aeson.Types
     , coerceFromJSONKeyFunction
     , mapFromJSONKeyFunction
 
+    -- *** Generic keys
+    , GToJSONKey()
+    , genericToJSONKey
+    , GFromJSONKey()
+    , genericFromJSONKey
+
     -- ** Liftings to unary and binary type constructors
     , FromJSON1(..)
     , parseJSON1
@@ -116,6 +125,7 @@ module Data.Aeson.Types
     , sumEncoding
     , unwrapUnaryRecords
     , tagSingleConstructors
+    , rejectUnknownFields
 
     -- ** Options utilities
     , SumEncoding(..)
@@ -123,6 +133,18 @@ module Data.Aeson.Types
     , camelTo2
     , defaultOptions
     , defaultTaggedObject
+
+    -- ** Options for object keys
+    , JSONKeyOptions
+    , keyModifier
+    , defaultJSONKeyOptions
+
+    -- * Parsing context
+    , (<?>)
+    , JSONPath
+    , JSONPathElement(..)
+    , formatPath
+    , formatRelativePath
     ) where
 
 import Prelude.Compat
