@@ -17,7 +17,7 @@ import Text.JSON
 instance NFData JSValue where
     rnf JSNull = ()
     rnf (JSBool b) = rnf b
-    rnf (JSRational b r) = rnf b `seq` rnf r `seq` ()
+    rnf (JSRational b r) = rnf b `seq` rnf r
     rnf (JSString s) = rnf (fromJSString s)
     rnf (JSArray vs) = rnf vs
     rnf (JSObject kvs) = rnf (fromJSObject kvs)
