@@ -1,6 +1,7 @@
+{-# LANGUAGE NoImplicitPrelude #-}
+
 module Typed.Generic (benchmarks, decodeBenchmarks) where
 
-import Prelude ()
 import Prelude.Compat
 
 import Data.Aeson hiding (Result)
